#include <Adafruit_NeoPixel.h>
#include <WiFi.h>
#include "time.h"


bool test = true;

const char* ssid       = "SSID_HERE";
const char* password   = "PASSWORD_HERE";

const char* ntpServer = "pool.ntp.org";
const long  gmtOffset_sec = -18000;
const int   daylightOffset_sec = 3600;

#define PIN        12  // Pin number connected to the NeoPixel strip
#define NUMPIXELS  56  // Number of LEDs in the strip

// Declare our NeoPixel strip object:
Adafruit_NeoPixel strip(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);

int max_brightness = 90;
int min_brightness = 0;

/*
algorithm:

24 hours * 60 minutes = 1440 mintues

N LEDs --- min_brightness <= value <= max_brightness

1440/N = M minutes per led

  1                   x
 --- = ----------------------------------
  M    (max_brightness-min_brightness)+1

x = dV (decrease value per 1 minute) = 146/M

formula:
dV = ((max_brightness-min_brightness)+1)/(1440/N) = ((max_brightness-min_brightness)+1)*N/1440

*/

int dV = 0;


///////// test clock /////////
int test_hour = 0;
int test_minute = 0;

// Function to update the clock
void updateTestClock() {
    test_minute++;
    if (test_minute == 60) {
        test_minute = 0;
        test_hour++;
        if (test_hour == 24) {
            test_hour = 0;
        }
    }

}
////////////////////////////////


void printLocalTime()
{
  struct tm timeinfo;
  if(!getLocalTime(&timeinfo)){
    Serial.println("Failed to obtain time");
    return;
  }
  Serial.println(&timeinfo, "%A, %B %d %Y %H:%M:%S");
}


void setColor(uint32_t color) {
  for (int i = 0; i < strip.numPixels(); i++) {
    strip.setPixelColor(i, color);
  }
  strip.show();
}

void showLocalTimeOnStrip () {
  struct tm timeinfo;
  if(!getLocalTime(&timeinfo)){
    Serial.println("Failed to obtain time");
    return;
  }
  int pV = 0;
  if (test){
  pV = ((60*test_hour) + test_minute) * dV ; //pV = passed value
  }else{
  pV = ((60*timeinfo.tm_hour) + timeinfo.tm_min) * dV ; //pV = passed value
  }
  Serial.println(pV);
  int offLEDs = pV / ((max_brightness-min_brightness)+1);
  int lastLEDBrightnessOffset = pV % ((max_brightness-min_brightness)+1);
  for(int i= NUMPIXELS; i>= 0;i--){
if (NUMPIXELS - 1 - offLEDs < i){
  strip.setPixelColor(i, strip.Color(min_brightness, min_brightness, min_brightness));
}else if (i== NUMPIXELS - 1 - offLEDs){
    strip.setPixelColor(i, strip.Color(max(0,max_brightness-(lastLEDBrightnessOffset)), max(0,max_brightness-(lastLEDBrightnessOffset*2)), max(0,max_brightness-(lastLEDBrightnessOffset*4))));
}else{
      strip.setPixelColor(i, strip.Color(max_brightness, max_brightness, max_brightness));
}
  }
  strip.show();
}


void setup() {

  Serial.begin(9600);
  dV = ((max_brightness-min_brightness)+1) * NUMPIXELS/1440;

  if (dV == 0){
    Serial.printf("dV is not calculated");
    return;
  }else{
    Serial.printf("dV is", dV);
  }

  //connect to WiFi
  Serial.printf("Connecting to %s ", ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
      delay(500);
      Serial.print(".");
  }
  Serial.println(" CONNECTED");
  
  //init and get the time
  configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);
  printLocalTime();

  //disconnect WiFi as it's no longer needed
  WiFi.disconnect(true);
  WiFi.mode(WIFI_OFF);
  
  strip.begin();           // Initialize the strip
  strip.show();            // Initialize all pixels to 'off'
  strip.setBrightness(max_brightness);
  strip.setPixelColor(strip.numPixels()-1,strip.Color(max_brightness,max_brightness,max_brightness));
    strip.show();           
  delay(1000);
}

void loop() {
  if(test){
    updateTestClock();
    Serial.printf("Test Clock : %i : %i\n",test_hour,test_minute);
  }else{
  printLocalTime();                // Hold for 3 seconds
  }
  showLocalTimeOnStrip();
  if (test){
  delay(100);
  }else{
    delay(1000);
  }
}

